"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveModule = exports.findPackageFileExtension = void 0;
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const debug_cjs_1 = require("../debug.cjs");
function getFile(def, wantModule) {
    if (typeof def === 'string')
        return def;
    if (wantModule) {
        return def.import || def.default;
    }
    else {
        return def.require || def.default;
    }
}
// const regExtension = /\.[^\.]+$/;
const prefixRelative = /^[\.\/]+/;
function match(path, file) {
    path = path.replace(prefixRelative, '');
    file = file.replace(prefixRelative, '');
    if (file.startsWith(path)) {
        return file.slice(path.length).replace(prefixRelative, '');
    }
    else {
        return null;
    }
}
function findPath(exportPaths, file) {
    for (const path of exportPaths) {
        if (file === path)
            return { index: path, file: '.' };
        if (path.endsWith('/')) {
            const m = match(path, file);
            if (m) {
                return { index: path, file: m };
            }
        }
    }
    return null;
}
function findPackageFileExtension(pkg, wantModule) {
    if (pkg.exports && pkg.exports['.']) {
        const ex = pkg.exports['.'];
        if (typeof ex === 'string') {
            return (0, path_1.extname)(ex);
        }
        if (wantModule && ex.import) {
            return (0, path_1.extname)(ex.import);
        }
        if (!wantModule && ex.require) {
            return (0, path_1.extname)(ex.require);
        }
        if (ex.default) {
            return (0, path_1.extname)(ex.default);
        }
    }
    if (pkg.module && wantModule) {
        return (0, path_1.extname)(pkg.module);
    }
    if (pkg.type !== 'module' && !wantModule && pkg.main) {
        return (0, path_1.extname)(pkg.main);
    }
    return '';
}
exports.findPackageFileExtension = findPackageFileExtension;
function resolveInPackage(pkgPath, ...seg) {
    const root = (0, path_1.resolve)(pkgPath, '..');
    const p = (0, path_1.resolve)(root, ...seg);
    if (p.startsWith(root)) {
        return p;
    }
    else {
        throw new Error('Resolve to outside package: ' + p);
    }
}
/**
 * resolve module file in target package
 * emulate nodejs version 14+'s
 * @param wantType resolve type, esm or cjs
 * @param packageJsonFilePath  absolute path to package's package.json
 * @param file resolve file inside package
 * @param useModuleField if true, use "module" field when package type=commonjs
 */
function resolveModule(wantType, packageJsonFilePath, file = '.', useModuleField = false) {
    const wantModule = wantType === 'module';
    if (!file) {
        file = '.';
    }
    if (!(0, fs_extra_1.pathExistsSync)(packageJsonFilePath)) {
        (0, debug_cjs_1.debugOut)('! missing file: %s', packageJsonFilePath);
        return null;
    }
    const pkg = require(packageJsonFilePath);
    if (pkg.exports) {
        const exportPaths = Object.keys(pkg.exports);
        const found = findPath(exportPaths, file);
        if (found) {
            const result = getFile(pkg.exports[found.index], wantModule);
            if (result) {
                const foundFile = found.file;
                if (foundFile !== '.') {
                    const extension = findPackageFileExtension(pkg, wantModule);
                    if ((0, path_1.extname)(file) !== extension) {
                        file += extension;
                    }
                }
                return resolveInPackage(packageJsonFilePath, result.replace(/^[\.\/]+/, ''), foundFile);
            }
        }
    }
    const isPackageDefaultCommonjs = pkg.type !== 'module';
    if (isPackageDefaultCommonjs) {
        if (wantModule) {
            if (!useModuleField) {
                return null;
            }
            if (file === '.') {
                if (pkg.module) {
                    return resolveInPackage(packageJsonFilePath, pkg.module);
                }
                return null;
            }
            if (pkg.module && pkg.main) {
                const ext = (0, path_1.extname)(pkg.module);
                const pmain = (0, path_1.dirname)(pkg.main).split('/').reverse();
                const pmdl = (0, path_1.dirname)(pkg.module).split('/').reverse();
                while (pmain[0] === pmdl[0] && pmdl.length > 0) {
                    pmain.shift();
                    pmdl.shift();
                }
                const moduleSubRoot = pmdl.reverse().join('/') || '.';
                if (!(0, path_1.extname)(file)) {
                    file += ext;
                }
                return resolveInPackage(packageJsonFilePath, moduleSubRoot, file);
            }
        }
        else {
            if (file === '.') {
                if (pkg.main) {
                    return resolveInPackage(packageJsonFilePath, pkg.main);
                }
            }
            else {
                const extension = findPackageFileExtension(pkg, false);
                if ((0, path_1.extname)(file) !== extension) {
                    file += extension;
                }
                return resolveInPackage(packageJsonFilePath, file);
            }
        }
    }
    return null; //
}
exports.resolveModule = resolveModule;
//# sourceMappingURL=resolve.simulate.js.map